import base64
import hashlib
from datetime import datetime, timedelta
from typing import Optional


def gen_hash(
    url: str, expires_timestamp: Optional[int], secret_key: str
) -> str:
    if expires_timestamp is None:
        timestamp_value = ''
    else:
        timestamp_value = expires_timestamp
    raw_value = '{expires_timestamp}{url} {secret_key}'.format(
        expires_timestamp=timestamp_value, url=url, secret_key=secret_key
    )
    md5_hash = hashlib.md5(raw_value.encode('utf-8')).digest()
    base64_token = (
        base64.urlsafe_b64encode(md5_hash)
        .decode('utf-8')
        .replace('=', '')
        .replace('/', '')
    )
    return base64_token


def gen_expires(dt: datetime, seconds: int) -> int:
    expires_timedelta = timedelta(seconds=seconds)
    expires_at = dt + expires_timedelta
    timestamp = int(expires_at.timestamp())
    return timestamp



from django.conf import settings
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Generates pre-signed url by passing public url'

    def add_arguments(self, parser):
        parser.add_argument('url', type=str, default=1)
        parser.add_argument('--seconds', type=int, default=1)

    def handle(self, *args, **options):
        seconds = options.get(
            'seconds', settings.SECURE_LINK_EXPIRATION_SECONDS
        )
        url = default_storage.pre_sign_url(options['url'], seconds)
        self.stdout.write(url)



import re

from django.conf import settings
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand
from django.template.loader import render_to_string

from nginx_secure_links import settings as app_settings


class Command(BaseCommand):
    help = 'Generates a sample of Nginx location basing on the settings'

    def handle(self, *args, **options):
        private_suffixes = []
        public_suffixes = []
        media_root = str(settings.MEDIA_ROOT).rstrip("/")
        media_location = settings.MEDIA_URL.replace(
            default_storage.not_hashable_prefix, ""
        ).rstrip("/")

        # prefixes preparing
        for prefix in app_settings.SECURE_LINK_PRIVATE_PREFIXES:
            private_suffixes.append(prefix.strip("/"))
        for prefix in app_settings.SECURE_LINK_PUBLIC_PREFIXES:
            public_suffixes.append(prefix.strip("/"))
        media_is_private = (
            len(public_suffixes) > 0 or len(private_suffixes) == 0
        )
        context = {
            'private_suffixes': private_suffixes,
            'public_suffixes': public_suffixes,
            'media_is_private': media_is_private,
            'media_location': media_location,
            'media_root': media_root,
            'token_field': app_settings.SECURE_LINK_TOKEN_FIELD,
            'expires_field': app_settings.SECURE_LINK_EXPIRES_FIELD,
            'secret': app_settings.SECURE_LINK_SECRET_KEY,
        }
        rendered_locations = render_to_string(
            'nginx_secure_links/locations.conf', context
        )
        rendered_locations = re.sub('\n\n+', '\n', rendered_locations).strip()
        self.stdout.write(rendered_locations)



import datetime
import operator
import re
import urllib.parse
from typing import Optional
from urllib.parse import urlparse

from django.core.exceptions import ImproperlyConfigured
from django.core.files.storage import FileSystemStorage
from django.utils.functional import cached_property

from nginx_secure_links import settings as app_settings
from nginx_secure_links import utils


class FileStorage(FileSystemStorage):
    def __init__(
        self,
        *args,
        expires_seconds=None,
        nginx_secret_key=None,
        token_field_name=None,
        expires_field_name=None,
        private_prefixes=None,
        public_prefixes=None,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        # secure links attributes
        self._expires_seconds = expires_seconds
        self._nginx_secret_key = nginx_secret_key
        self._token_field_name = token_field_name
        self._expires_field_name = expires_field_name
        self._private_prefixes = private_prefixes
        self._public_prefixes = public_prefixes

    @cached_property
    def expires_seconds(self) -> int:
        return self._value_or_setting(
            self._expires_seconds, app_settings.SECURE_LINK_EXPIRATION_SECONDS
        )

    @cached_property
    def nginx_secret_key(self) -> str:
        return self._value_or_setting(
            self._nginx_secret_key,
            app_settings.SECURE_LINK_SECRET_KEY,
        )

    @cached_property
    def token_field_name(self) -> str:
        return self._value_or_setting(
            self._token_field_name,
            app_settings.SECURE_LINK_TOKEN_FIELD,
        )

    @cached_property
    def expires_field_name(self) -> str:
        return self._value_or_setting(
            self._expires_field_name,
            app_settings.SECURE_LINK_EXPIRES_FIELD,
        )

    @cached_property
    def private_prefixes(self) -> list:
        return self._value_or_setting(
            self._private_prefixes,
            app_settings.SECURE_LINK_PRIVATE_PREFIXES,
        )

    @cached_property
    def public_prefixes(self) -> list:
        return self._value_or_setting(
            self._public_prefixes,
            app_settings.SECURE_LINK_PUBLIC_PREFIXES,
        )

    @cached_property
    def prefixes(self) -> list:
        if len(self.private_prefixes) > 0 and len(self.public_prefixes) > 0:
            raise ImproperlyConfigured

        return self.private_prefixes or self.public_prefixes or []

    @cached_property
    def not_hashable_prefix(self) -> str:
        """
        When `base_url` contains schema://domain:port/media/, only
        url path should be hashed.
        The method helps to calculate which prefix should not be hashed for
        further logic.
        """
        schema_prefix = r'^http(s)?://'
        if re.match(schema_prefix, self.base_url, re.I):
            o = urlparse(self.base_url)
            path_start_index = o.geturl().index(o.path)
            return self.base_url[:path_start_index]
        return ''

    @cached_property
    def prefix_predicate(self) -> callable:
        """
        Build an operator function which should be applied on the condition
         like "path in prefixes" inside `_is_secure_path(...)`.
        Since we use `self.prefixes` as a single property which optionally
        has private or public prefixes as a result (not both),
        we need to have a predicate for "path in self.prefixes" condition.
        The predicate is being used as a check "if path should be hashed".

        Cases:
         - private_prefixes specified:
             prefix in private_prefixes- should be hashed (True)
         - public_prefixes specified:
             prefix in public_prefixes- should not be hashed (False)
         - private_prefixes and public_prefixes are equal to "[]":
             should be always hashed, regardless of the condition result (True)
        """
        if len(self.private_prefixes) > 0:
            return operator.truth
        elif len(self.public_prefixes) > 0:
            return operator.not_
        return lambda x: True

    def _is_secure_path(self, url: str) -> bool:
        """Checks if the `url` should be hashed"""
        # shortcut: should not be iterating because of all urls will be private
        if len(self.prefixes) == 0:
            return True
        any_prefix_matched = any(
            (
                url.startswith(
                    '{base_url}{prefix}'.format(
                        base_url=self.base_url, prefix=prefix
                    )
                )
                for prefix in self.prefixes
            )
        )
        return self.prefix_predicate(any_prefix_matched)

    def url(self, name: str, lifetime: int = None) -> str:
        if lifetime is None:
            # default lifetime
            expires_seconds = self.expires_seconds
        elif lifetime == 0:
            # unlimited lifetime
            expires_seconds = None
        else:
            expires_seconds = lifetime
        url = super().url(name)
        return self.pre_sign_url(url, seconds=expires_seconds)

    def pre_sign_url(self, url: str, seconds: Optional[int]) -> str:
        """Generates pre-signed url with md5 token"""
        # skip if it's not specified in `self.path_prefixes`
        if not self._is_secure_path(url):
            return url

        hashable_url = url.replace(self.not_hashable_prefix, '')

        if seconds is not None:
            current_time = datetime.datetime.now(tz=datetime.timezone.utc)
            timestamp = utils.gen_expires(current_time, seconds)
        else:
            timestamp = None
        token = utils.gen_hash(hashable_url, timestamp, self.nginx_secret_key)
        params = urllib.parse.urlencode(
            {
                self.token_field_name: token,
                self.expires_field_name: timestamp or '',
            }
        )
        return '{url}?{params}'.format(url=url, params=params)



# -*- coding: utf-8 -*-
import os

from setuptools import setup

version = __import__('nginx_secure_links').__version__

long_description = """django-nginx-secure-links extends django
 FileSystemStorage by implementing secure links based on temporal links and uses
 Nginx secure links module for that. See the project page for more information:
  http://github.com/lighTechLLC/django-nginx-secure-links"""

if os.path.isfile('README.rst'):
    with open('README.rst') as f:
        long_description = f.read()

setup(
    name='django-nginx-secure-links',
    version=version,
    description="Django storage based on Nginx secure links module",
    long_description_content_type='text/x-rst',
    long_description=long_description,
    author='Eugene Hatsko',
    author_email='ehatsko@gmail.com',
    maintainer='Eugene Hatsko',
    maintainer_email='ehatsko@gmail.com',
    url='http://github.com/lighTechLLC/django-nginx-secure-links',
    license='MIT License',
    platforms=['any'],
    packages=[
        'nginx_secure_links',
    ],
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=["Django>=3.2"],
    extras_require={},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    project_urls={
    },
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

SECURE_LINK_EXPIRATION_SECONDS = getattr(
    settings, 'SECURE_LINK_EXPIRATION_SECONDS', 60 * 60 * 24
)
SECURE_LINK_TIMEZONE = getattr(settings, 'SECURE_LINK_TIMEZONE', 'UTC')
SECURE_LINK_SECRET_KEY = getattr(settings, 'SECURE_LINK_SECRET_KEY')
SECURE_LINK_TOKEN_FIELD = getattr(settings, 'SECURE_LINK_TOKEN_FIELD', 'token')
SECURE_LINK_EXPIRES_FIELD = getattr(
    settings, 'SECURE_LINK_EXPIRES_FIELD', 'expires'
)
SECURE_LINK_PRIVATE_PREFIXES = getattr(
    settings, 'SECURE_LINK_PRIVATE_PREFIXES', []
)
SECURE_LINK_PUBLIC_PREFIXES = getattr(
    settings, 'SECURE_LINK_PUBLIC_PREFIXES', []
)
if (
    len(SECURE_LINK_PRIVATE_PREFIXES) > 0
    and len(SECURE_LINK_PUBLIC_PREFIXES) > 0
):
    raise ImproperlyConfigured(
        'Only SECURE_LINK_PRIVATE_PREFIXES or SECURE_LINK_PUBLIC_PREFIXES '
        'should be specified, not both'
    )
